
# OKTA Constants
OKTA_SESSION_DURATION = 43200
SUPPORTED_OKTA_FACTOR_TYPES = ['GOOGLE', 'OKTA']

FIGGY_KEYRING_NAMESPACE = "figgy"
FIGGY_KEYRING_ASSERTION_NAME = "okta_assertion"

SAML_ASSERTION_CACHE_KEY = 'assertion'
SAML_ASSERTION_MAX_AGE = 60 * 5 * 1000  # Assertions don't last long, 5 mins max reuse
